class Constants:
	TARGET_CASE_IDX = "custom_column_case_idx"
	TARGET_ACTIVITY = "custom_column_activity"
	TARGET_ACTIVITY_CODE = "custom_column_activity_code"
	TARGET_TIMESTAMP = "custom_column_timestamp"
	TARGET_EV_IDX = "custom_column_event_idx"
	TARGET_EV_CASE_MULT_ID = "custom_column_ev_case_mult_idx"
	CASE_DURATION = "custom_column_case_duration"

	TARGET_PRE_ACTIVITY = "custom_column_pre_activity_code"
	TARGET_PRE_TIMESTAMP = "custom_column_pre_timestamp"
	TARGET_PRE_CASE = "custom_column_pre_case"
	TARGET_VARIANT_NUMBER = "custom_column_variant_number"
	TIMESTAMP_DIFF = "custom_column_timestamp_diff"

	VARIANTS_SEP = ","
	CUSTOM_PREFIX = "custom_column"
	TEMP_COLUMN_1 = "custom_column_temp_1"
	TEMP_COLUMN_2 = "custom_column_temp_2"
